;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x85009bc1b6ab586ab602e171206a27c3 . "./code.cg2js/cg-acache.fasl")
(#x729050e6cd5b01527fda233f271503e4 . "./code.cg2js/cg-base.fasl")
(#xfc959f0aa7132a5c57a975a7b8a21ab4 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x9db7d2e50c9d485019d8f5f7c66bd89e . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdc35f7f07d94912ad36a4887352682c3 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xbf7da5a2a2b783f87c3ef723af8693e5 . "./code.cg2js/cg-button.fasl")
(#xfed2372dc2e3daf4078b1aaa7f135649 . "./code.cg2js/cg-calendar.fasl")
(#x82d352e295324aea5c4a180e50fcadbc . "./code.cg2js/cg-caret.fasl")
(#x383a90f284d30135ac640eb7156db00a . "./code.cg2js/cg-chart-or-plot.fasl")
(#x328555b341d5f9503963f833d78b4eab . "./code.cg2js/cg-chart-widget.fasl")
(#x91b5cbcb03a2aa2885e44f95142fa1d7 . "./code.cg2js/cg-check-box.fasl")
(#x47d9c357fc83091c6466ceecde6d8963 . "./code.cg2js/cg-choice-list.fasl")
(#xde52eea76cb8924d0ea4f35c17710ffd . "./code.cg2js/cg-choose-printer.fasl")
(#x9b8457afe63846f7943c8f0a1e3a8cff . "./code.cg2js/cg-class-grid.fasl")
(#x21bf2805ed917db4fcc3c67c8ac1fdd9 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xcf667b65546eac539458a07dd8d45d15 . "./code.cg2js/cg-class-support.fasl")
(#x1b079a7befaff3d81f73bb764f4331b7 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xcea189a8b39475995f074ace9981e854 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x51691487f2f8957c430c21f06c860e3e . "./code.cg2js/cg-clipboard.fasl")
(#x58a4453b9708d76b9f07ad20bad41881 . "./code.cg2js/cg-color-dialog.fasl")
(#xebe8f185a9faf1544aba98e776b38c04 . "./code.cg2js/cg-combo-box.fasl")
(#xcfd22551cdaf4ae36b3e17e5aeacea48 . "./code.cg2js/cg-common-control.fasl")
(#x480f3b30a58fc6e4530d2cb55f0a9c25 . "./code.cg2js/cg-comtab.fasl")
(#x8c89381db34fb2422b714939a38e5996 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x573571e977464d79d4f87b59f98374a5 . "./code.cg2js/cg-curve.fasl")
(#x46c5ad777d79b9ba9fa05c0b322ba7f4 . "./code.cg2js/cg-dde-utils.fasl")
(#x3f25516a7f163a4e4eb3761b9605770d . "./code.cg2js/cg-dev.fasl")
(#x7e8d27a99f2176d1705f8adef74d95c4 . "./code.cg2js/cg-dialog-item.fasl")
(#xea7ce8db02ab1465f6915773708f360d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb276ef13dd68a9e837f43adbbc2d9808 . "./code.cg2js/cg-directory-dialog.fasl")
(#xe99fc57e563c197f5480f0449d68df2b . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x7c333877534f724e11aa4cb0b7bd8f34 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x14f4dd2d9f6e8e13cca7380f9dd13989 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xdb7049c7590669d78f07e393d5688acb . "./code.cg2js/cg-drawable.fasl")
(#x3fc40b12ee76c31e130bcbeddf8ae879 . "./code.cg2js/cg-dropping-outline.fasl")
(#xd8a79ce3e3afb11aaeb472b05da2616a . "./code.cg2js/cg-edit-in-place.fasl")
(#x17847c2f8ff7bed9a893911f482b5890 . "./code.cg2js/cg-editable-text.fasl")
(#x1d8d40ecd59b16f7b54125fb1d880ec2 . "./code.cg2js/cg-file-dialog.fasl")
(#x43e182b78c5f2ef3fc7ed99810990d5c . "./code.cg2js/cg-fill-texture.fasl")
(#x3528dbcb118e34795c1a4e77545fced5 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x316cb069c5415407e1c66494bc60fa44 . "./code.cg2js/cg-font-dialog.fasl")
(#x9db4269ecee4075f36bc731ad2de00ed . "./code.cg2js/cg-gesture-emulation.fasl")
(#x2cef8096e042f347c53c4b6d5137aff1 . "./code.cg2js/cg-get-pixmap.fasl")
(#x87a2007248cebd8d95cc57c929a06887 . "./code.cg2js/cg-get-position.fasl")
(#x5e971c59408fefa1c4f84086b01f9f01 . "./code.cg2js/cg-graphics-context.fasl")
(#x12d77f0e58f02f357880b3c850ce470b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xe971eaf8e6e2301ad08bf44cf60e9479 . "./code.cg2js/cg-grid-widget.fasl")
(#x9564c46db4be63a51aeb8d51a4fe19df . "./code.cg2js/cg-group-box.fasl")
(#x46ebeec6338d439073a3d1cac6cb87e3 . "./code.cg2js/cg-header-control.fasl")
(#xe48d6eb4c8cea870e26656ab40124a7a . "./code.cg2js/cg-hotspot.fasl")
(#x79fc9c196b8032d5accfb1a7e1c09b83 . "./code.cg2js/cg-html-dialog.fasl")
(#xc48d535744785545d15391e7a5bc1b97 . "./code.cg2js/cg-html-widget.fasl")
(#x59fedfd3ce98036446cb6630845b1150 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x48daae77f8770021d77d689ad39cc240 . "./code.cg2js/cg-icon.fasl")
(#x1cffd2f05a514146e58c3da30a41edd1 . "./code.cg2js/cg-ie.fasl")
(#x0f21bc6ee755eaea7ed43e67d0a63f93 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x08a0c324f23f28f9f44e0ab19cc5d92e . "./code.cg2js/cg-intersect.fasl")
(#x31373dfc51b8d62d7d89dd0c9e3aeb89 . "./code.cg2js/cg-item-list.fasl")
(#xe3b7acbefae92357f56bd2ef49207abe . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x5e4381a47365118832742cfe6107d630 . "./code.cg2js/cg-lamp.fasl")
(#x7c7d5fda6c2c7aabc1838d2885f617f1 . "./code.cg2js/cg-layout.fasl")
(#x356cc317859df661cbdf04dae255ea50 . "./code.cg2js/cg-lettered-menu.fasl")
(#x65af16d78018c38d4cb45bfc18889b74 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd06632e03a0b83b592bb5367bc1c8dbb . "./code.cg2js/cg-lisp-text.fasl")
(#xdfc6268e17fbd0243535655d17cc2c60 . "./code.cg2js/cg-lisp-widget.fasl")
(#x93b34fc2ca2ae16a054d4a8ce2940e4b . "./code.cg2js/cg-list-view.fasl")
(#x1d04fe64b8da1ac1915581e74962c774 . "./code.cg2js/cg-mci.fasl")
(#xda6570fa62e2c67d6f0bc649773873a6 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x044999f517849f584f20a39dcdb0042c . "./code.cg2js/cg-menu.fasl")
(#xe53625c957e05f3854ca21a3440772f9 . "./code.cg2js/cg-message-dialog.fasl")
(#x56adc268139f62a736083763d5ea58fc . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x73a2caf4154f498ea089ab4b5050250c . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xad599e1259e7217a54fa199974fa79b8 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x186529822586d219e1d50420ab77ddcf . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xb4b68f0241fcefa0928941669848b75b . "./code.cg2js/cg-multi-picture-button.fasl")
(#x47dc892622ae45c5624aba5fbe9ca135 . "./code.cg2js/cg-nodes.fasl")
(#x82623157213fcf0d432883df8dcfbd20 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xbe14b06679620dd5dfb372a3b181e64f . "./code.cg2js/cg-object-editor.fasl")
(#x12e7dea081dd8aa02d6cc6909fa8f1bb . "./code.cg2js/cg-ocx.fasl")
(#xd216d9cc242770c7d8d6572c23e114e5 . "./code.cg2js/cg-os-widget.fasl")
(#x953f1f3ab3843c34c7ff77c70d4c3d60 . "./code.cg2js/cg-os-window.fasl")
(#xc6818f4722dbb0111e4f8aec56bca5b5 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x860d8ee8cc4c6e353e4683a41d7eb164 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x039d9c7e2346fa4a9115541a59e60e0d . "./code.cg2js/cg-outline.fasl")
(#x9c3f34f418d25408bd517c709723bdad . "./code.cg2js/cg-palette.fasl")
(#x25acbb8787dc045e12c16c018eb29450 . "./code.cg2js/cg-paren-matching.fasl")
(#x808b7661eaae0c111f837e74578d45f2 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xa510bcb92489e2c34ed8f9422f10d83d . "./code.cg2js/cg-picture-widget.fasl")
(#xa2f5455639c17160a8ed4fe1e77034c3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x4e8e0fef0f8a9c1fec4800b87e1996d5 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xea0dfe4c2c9ff0929075abbef814bef1 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x1d059a283f3bb41a8653091ee8ed4a51 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xfb6f0f3bfebf54abddd22d156261e7a9 . "./code.cg2js/cg-pixmap.fasl")
(#x24e958dba77bd3eeddfaa1f311d357e6 . "./code.cg2js/cg-plot-widget.fasl")
(#x9d4ee47df087648f089455a00dfeb403 . "./code.cg2js/cg-printing.fasl")
(#xcbe17997ddebb777ff6eb0f087801734 . "./code.cg2js/cg-progress-indicator.fasl")
(#xb1abae4b317c3b6502c8ffc8405b9033 . "./code.cg2js/cg-project-window.fasl")
(#xddcbfa39f3cde4fbabcbeda774342b99 . "./code.cg2js/cg-property.fasl")
(#xa74e5fd03c64099bcd0be6ddd640277e . "./code.cg2js/cg-radio-button.fasl")
(#x2b96d7c0ed8837fe70d7be895acf0d9b . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x31f86a42259cf312cfc3931377f96264 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x16a20ab0d8cf954e59d7167feaf0e2f2 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x9ed8894c435481be8f8d0df1c03fc80b . "./code.cg2js/cg-rich-edit.fasl")
(#xb1af7cd160b86ea23a125a354a315193 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x0d788488fdfda95f9b93f27847bfeb57 . "./code.cg2js/cg-scaling-stream.fasl")
(#xd2898e7c1fc1072bdaf31485194ace86 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xa1da518f537b79abaf635f31854127ac . "./code.cg2js/cg-scroll-bar.fasl")
(#xfa57ebbbabc1e94122ed2287635b5361 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x0b68559c3afc376c7bddeeeb2d9b1985 . "./code.cg2js/cg-selected-object.fasl")
(#xd3f4a97083de2cd0b75d4a0776c0eaba . "./code.cg2js/cg-shortcut-menu.fasl")
(#x713ec96d35e811e71a80885b1ad51035 . "./code.cg2js/cg-split-bar.fasl")
(#x4fdf32651b0588ff9f817f9e8753cd40 . "./code.cg2js/cg-static-text.fasl")
(#xaebea86632169d03b947500eda086347 . "./code.cg2js/cg-status-bar.fasl")
(#xaafedad90c19c0b68ce7771303b8fadf . "./code.cg2js/cg-string-dialog.fasl")
(#xa349bfba11121561aacf68f5b208c065 . "./code.cg2js/cg-tab-control.fasl")
(#x9c033f525b96f1f17d16438f9610087e . "./code.cg2js/cg-template-string.fasl")
(#x0c53ca48f67e6a2bb58824d640bf21df . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x82f0bbde3a922d50183225c0ed581a66 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa16461825efe4147ae71ba02a9bd4509 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x6b8ca6a21f526ecb0e0295e7f78c6276 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8060358dd82c7ae21fd47e9419163662 . "./code.cg2js/cg-text-widget.fasl")
(#xccdbbdec11dab2209ce5a90faaf3f74a . "./code.cg2js/cg-timer.fasl")
(#x4f8127450149191b68a9e454c6c2e233 . "./code.cg2js/cg-toggling-widget.fasl")
(#x561027ac933281942588fab5d405c3ec . "./code.cg2js/cg-toolbar.fasl")
(#xba1dc5bb0b69ead066611f613a921bce . "./code.cg2js/cg-tooltip.fasl")
(#xff56183b49b5487b5650c0c3933b6a2d . "./code.cg2js/cg-touch.fasl")
(#xc3eccf501184258ab71db0f308d9079c . "./code.cg2js/cg-trackbar.fasl")
(#x7a66baa292db1c323b3187ec119cc4af . "./code.cg2js/cg-tray.fasl")
(#xd252d1a1305927f542e6ce2c9e1fed4b . "./code.cg2js/cg-up-down-control.fasl")
(#x7a09b4e106abb2924e2dbe25149d9c88 . "./code.cg2js/cg-utility-dialog.fasl")
(#x45d1081569b5c957034a40df97cbf5aa . "./code.cg2js/cg-web-browser-dde.fasl")
(#xc7bf34482f6640d7ea8ac12d349c12d8 . "./code.cg2js/cg-web-browser.fasl")
(#xb1222e96bc9847b20b8160212cfcc52a . "./code.cg2js/cg-wrap-string.fasl")
(#x118613a9012b3ab07cc75d423e2fc6cb . "./code.cg2js/cg-yes-no-list.fasl")
(#x856c5203994863dcd7fe34c7c42760df . "./code.cg2js/cg-yes-no-string.fasl")
(#x7092b7e85566ae3c5d7615d6aa6745a9 . "./code.cg2js/cg.fasl")
(#xbae929cd25b51beaef4cb02de14834a0 . "./code.idejs/ide-acache.fasl")
(#x7c62c2f2ffd78800e5e63ee241dd0b72 . "./code.idejs/ide-apropos.fasl")
(#x29fb1998ab753344242c57970bc6d4df . "./code.idejs/ide-base.fasl")
(#xa1c7b5d246165b9384eff8b44639532d . "./code.idejs/ide-class-browser.fasl")
(#x7a1d848f303beaef0c9b5351d690f167 . "./code.idejs/ide-class-grapher.fasl")
(#x39ba4f437707481c8cc33b0b65bc4f4f . "./code.idejs/ide-clipboard.fasl")
(#x691fe858ac92d0efca9f47319617304a . "./code.idejs/ide-dev.fasl")
(#x654374c76ae15217adf51932be6924d6 . "./code.idejs/ide-editor.fasl")
(#x4dda391d962c2e508dc063c1ac274a55 . "./code.idejs/ide-find-definitions.fasl")
(#x633d5fced7678a0815c06386680715e4 . "./code.idejs/ide-find-in-files.fasl")
(#x3e152f6d3d603a9da7cdf43de5355c7f . "./code.idejs/ide-form.fasl")
(#xb0113411a16fa577f0e4c58af4297ffe . "./code.idejs/ide-grapher.fasl")
(#xff2b3282afa99a736182caad1bd751e3 . "./code.idejs/ide-inspector.fasl")
(#x72816c0ac48ee8bec05c86b069828b3c . "./code.idejs/ide-intro.fasl")
(#xff3222c84a97bf0ec956e7f71b9620da . "./code.idejs/ide-keyshort.fasl")
(#x94c352a6312ea39f5a7a87bc965e0b17 . "./code.idejs/ide-navigator.fasl")
(#x9f1ff4a18cd31078e1257b93e5fad206 . "./code.idejs/ide-package-list.fasl")
(#xadb1ab9fb793b42b18b384ca611140d4 . "./code.idejs/ide-patch-dialog.fasl")
(#xd4cf1610b459fc9fa55bb2cbc939bcf1 . "./code.idejs/ide-process-browser.fasl")
(#x4704ca306485af6a9dfc23f338a53c1f . "./code.idejs/ide-project.fasl")
(#xfc4875746d6784940c108f11ee8417e2 . "./code.idejs/ide-trace.fasl")
(#xf57866bb722fd4a4e062bc2531d459fa . "./code.idejs/ide-window-list.fasl")
(#xf65c4341c8c7f8a94cc729b7a79a1157 . "./code.idejs/ide.fasl")
(#xea5d13ce0f9a2cfde1ff539415770ab2 . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
